/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 * 
 * http://tnc.inform.fh-hannover.de/
 */

#include "TNCUtil.h"

namespace TNCUtil {

	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *      void TNCUtil::writeToArray(char*, const int, const char*, ...)       *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void writeToArray(char *buf, const int size, const char *format, ...) {
	    va_list l;
	    va_start(l, format);
	    vsnprintf(buf, size, format, l);
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *  std::vector<std::string>* TNCUtil::getTokens(const char*, const char*)   *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::vector<std::string> * getTokens(const char *text, const char *delimiters) {
	    std::string data(text);
	
	    // Skip delimiters at beginning.
	    std::string::size_type lastPos = data.find_first_not_of(delimiters, 0);
	
	    // Find first "non-delimiter".
	    std::string::size_type pos = data.find_first_of(delimiters, lastPos);
	    std::vector<std::string> *tokens = new std::vector<std::string>();
	    while (std::string::npos != pos || std::string::npos != lastPos) {
	        // Found a token, put it into the vector
	        tokens -> push_back(data.substr(lastPos, pos - lastPos));
	
	        // Skip delimiters.
	        lastPos = data.find_first_not_of(delimiters, pos);
	
	        // Find next "non-delimiter"
	        pos = data.find_first_of(delimiters, lastPos);
	    }
	    return tokens;
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *                       void printWide(wchar_t*, int)                       *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void printWide(wchar_t *string, int length){
	    for (int i = 0; i < length*2 ; i+= 2) {
	        std::cout << ((unsigned char *)string)[i];
	    }
	    std::cout << std::endl;
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *                  unsigned short byteSpap(unsigned short)                  *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	unsigned short byteSwap(unsigned short nValue)
	{
	   return (((nValue>> 8)) | (nValue << 8));
	
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *                unsigned long byteSwap4Byte(unsigned long)                 *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	unsigned long byteSwap4Byte (unsigned long nLongNumber)
	{
	   return (((nLongNumber&0x000000FF)<<24)+((nLongNumber&0x0000FF00)<<8)+
	   ((nLongNumber&0x00FF0000)>>8)+((nLongNumber&0xFF000000)>>24));
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *     TNC_MessageType fromMessageType(TNC_VendorID, TNC_MessageSubtype)     *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	TNC_MessageType formMessageType(TNC_VendorID vendorID, TNC_MessageSubtype subType) {
	    TNC_MessageType messageType = (vendorID << 8) | (subType & 0xff);
	    return messageType;
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *           TNC_MessageSubtype enclosing_method(TNC_MessageType)            *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	TNC_MessageSubtype parseMessageSubType(TNC_MessageType messageType) {
	    TNC_MessageSubtype subType = 0x000000ff & messageType;
	    return subType;
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *            TNC_VendorID parseMessageVendorId(TNC_MessageType)             *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	TNC_VendorID parseMessageVendorId(TNC_MessageType messageType){
	    TNC_VendorID vendorId = messageType >> 8;
	    return vendorId;
	}

	#ifdef __WINDOWS__
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *               std::string TNCUtil::convertWCharString(BSTR)               *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::string convertWCharString(BSTR str) {
	    short length = wcslen(str) + 1;
	    char * buf = (char *) calloc(sizeof(char), length);
	    for (int i = 0; i < length * 2; i += 2) {
	        buf[i / 2] = str[i];
	    }
	    std::string ret(buf);
	    free(buf);
	    return ret;
	}
	#endif

}  // namespace TNCUtil
