/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#include "Exception.h"
#include "../TNCUtil.h"

#include <iostream>

namespace TNCUtil
{

namespace exception
{

#define MAX_MESSAGE_LENGTH 65536

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     Exception::Exception(std::string)                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Exception::Exception(std::string message)
	: message(message)
{
//	std::cout << "Exception::Exception(string)" << std::endl;
	// intentionally left blank
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     Exception::Exception(std::string)                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Exception::Exception(const char *formatString, ...)
{
//	std::cout << "Exception::Exception(...)" << std::endl;
	MACRO_TNCUtil_Exception_setString(formatString)
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                  Exception::Exception(const Exception &)                  *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Exception::Exception(const Exception & e)
	:message(e.message)
{
//	std::cout << "Exception::Exception(copy)" << std::endl;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                           Exception::~Exception()                         *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Exception::~Exception()
{
//	std::cout << "Exception::~Exception" << std::endl;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                   std::string Exception::getMessage()                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
std::string Exception::getMessage()
{
	return this->message;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *             void Exception::setString(const char * *, va_list)            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void Exception::setString(const char * formatString, va_list va)
{
	char *buf = (char *) calloc(MAX_MESSAGE_LENGTH, sizeof(char));
	vsnprintf(buf, MAX_MESSAGE_LENGTH, formatString, va);
	this->message = TNCUtil::toString(buf);
	free(buf);
}


} // exception

} // TNCUtil
