/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef EXCEPTION_H_
#define EXCEPTION_H_

#include <string>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

namespace TNCUtil
{

namespace exception
{

/**
 * This class encapsulates basic information (as string) when an exception has occured.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>18.02.2008 - create class (ib)</li>
 *   <li>15.04.2008 - fixed va_list [bug 8] (mbs)</li>
 * </ul>
 *
 * @class Exception
 * @brief Main class for exception handling.
 * @date 18.02.2008
 * @author Ingo Bente (ib)
 */
class Exception
{
public:

	/**
	 * Ctor.
	 *
	 * @param message A message (as std::string) providing information regarding the exception.
	 */
	Exception(std::string message);

	/**
	 * Ctor.
	 *
	 * @param formatString A message (as printf-like format string) providing information regarding the exception.
	 */
	Exception(const char* formatString, ...);

	/**
	 * Copy ctor.
	 *
	 * @param e The exception to be copied.
	 */
	Exception(const Exception & e);

	/**
	 * Dtor.
	 */
	virtual ~Exception();

	/**
	 * Getter method for the encapsulated message.
	 *
	 * @return The contained message describing the exception.
	 */
	std::string getMessage();

protected:

	/**
	 * Setter method for the message string. formatString is parsed using macros from stdarg.h.
	 */
	void setString(const char * formatString, va_list va);
#define MACRO_TNCUtil_Exception_setString(formatString) va_list va; va_start(va, formatString); setString(formatString, va); va_end(va);

private:
	std::string message;
};

}

}

#endif /*EXCEPTION_H_*/
