/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */
#ifndef RESULTEXCEPTION_H_
#define RESULTEXCEPTION_H_

#include "Exception.h"
#include "../tcg/tnc/tncifimc.h"
#include "../tcg/tnc/tncifimv.h"

namespace TNCUtil
{

namespace exception
{

/**
 * Inherits from class TNCUtil::exception::Exception. Contains additional information as TNC_Result.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>07.02.2008 - create class (mbs)</li>
 *   <li>18.02.2008 - changed to ResultException (ib)</li>
 * </ul>
 *
 * @class ResultException
 * @brief An exception class with TNC_Result
 * @date 07.02.2008
 * @author Mike Steinmetz (mbs)
 */
class ResultException : public Exception
{
public:

	/**
	 * Ctor.
	 *
	 * @param message A message (as std::string) providing information regarding the exception.
	 * @param returnValue A TNC_Result value that is of importance regarding the ResultException.
	 *
	 * @todo Change order of parameters to comply with other ctor.
	 */
	ResultException(std::string message, TNC_Result returnValue);

	/**
	 * Ctor.
	 *
	 * @param returnValue A TNC_Result value that is of importance regarding the ResultException.
	 * @param formatString A message (as printf-like format string) providing information regarding the exception.
	 */
	ResultException(TNC_Result returnValue, const char *formatString, ...);

	/**
	 * Destructor
	 */
	virtual ~ResultException();

	TNC_Result getReturnValue() const;

private:
	/**
	 * The returnvalue for TNCC or TNCS.
	 */
	TNC_Result returnValue;
};

} // namespace exception

} // namespace TNCUtil

#endif /*IMUNITEXCEPTION_H_*/
