/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 * 
 * http://tnc.inform.fh-hannover.de/
 */

#include "AbstractIMUnit.h"

namespace TNCUtil {

namespace iml {

// Classname for debugging and logging
#define CLASSNAME "AbstractIMUnit"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *        AbstractIMUnit::AbstractIMUnit(TNC_ConnectionID conID)             *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
AbstractIMUnit::AbstractIMUnit(TNC_ConnectionID conID)
	:connectionID(conID)
{
	TNCLog::getLog(CLASSNAME, "Constructor");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     AbstractIMUnit::~AbstractIMUnit()                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
AbstractIMUnit::~AbstractIMUnit()
{
	TNCLog::getLog(CLASSNAME, "~");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *        TNC_ConnectionState AbstractIMUnit::getConnectionState() const     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_ConnectionState AbstractIMUnit::getConnectionState() const {
    return connectionState;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *        TNC_ConnectionID AbstractIMUnit::getConnectionID() const     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_ConnectionID AbstractIMUnit::getConnectionID() const
{
    return connectionID;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *         void AbstractIMUnit::setConnectionState(TNC_ConnectionState)      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result AbstractIMUnit::notifyConnectionChange(TNC_ConnectionState newState)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "notifyConnectionChange");
    log.debug("New connection state: %d", newState);
    
    // check newState value
    if(newState >= 0 && newState <= 5){
    	connectionState = newState;
    
    	// call hook method for event notification
    	return notifyConnectionChange(); 	
    } else {
    	log.error("Invalid connection state: %d", newState);
    	return TNC_RESULT_INVALID_PARAMETER;
    }
    
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *            TNC_Result AbstractIMUnit::notifyConnectionChange()            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result AbstractIMUnit::notifyConnectionChange()
{
	return TNC_RESULT_SUCCESS;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *               TNC_Result AbstractIMUnit::batchEnding()                    *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result AbstractIMUnit::batchEnding()
{
	return TNC_RESULT_SUCCESS;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Result AbstractIMUnit::receiveMessage(TNC_BufferReference message,    *
 *     TNC_UInt32 messageLength, TNC_MessageType messageType)                *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result AbstractIMUnit::receiveMessage(TNC_BufferReference message, TNC_UInt32 messageLength, TNC_MessageType messageType)
{
	return TNC_RESULT_SUCCESS;
}

} // namespace iml

} // namespace TNCUtil
