/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */
#ifndef IFIMUNITIMPL_H_
#define IFIMUNITIMPL_H_

#include "../tcg/tnc/tncifimc.h"
#include "../tcg/tnc/tncifimv.h"
#include "../tcg/tnc/tncimunit.h"
#include "AbstractIMC.h"
#include "AbstractIMV.h"
#include <semaphore.h>
#include "../exception/ResultException.h"


namespace TNCUtil {

/** @file
 * IMunit specific functions.
 *
 * <h3>Changelog:</h3>
 * <ul> *
 *   <li>07.02.2008 - create class (mbs)</li>
 *   <li>06.01.2009 - added comment (ib)</li>
 * </ul>
 *
 * @date 07.02.2008
 * @author Mike Steinmetz (mbs)
 * @author Ingo Bente (ib)
 */
namespace iml {

// Version number of supported TNC IF-IMV specification
#define TNC_IFIMV_VERSION TNC_IFIMV_VERSION_1

// Version number of supported TNC IF-IMC specification
#define TNC_IFIMC_VERSION TNC_IFIMC_VERSION_1

/**
 * Initializes the IMC or IMV.
 */
TNC_Result TNC_IMUnit_Initialize(TNC_IMUnitID pImunitID, TNC_Version minVersion, TNC_Version maxVersion, TNC_Version ifIMUnitVersion, TNC_Version *pOutActualVersion) throw (TNCUtil::exception::ResultException);

/**
 * TODO_Comment
 *
 * @param enclosing_method_arguments
 *
 * @return return_type
 * @TODO delete or find out if it could be used.
 */
//TNC_Result TNC_IMUnit_NotifyConnectionChange(TNC_IMUnitID pImunitID, TNC_ConnectionID connectionID, TNC_ConnectionState newState) throw (TNCUtil::exception::ResultException);

/**
 * TODO_Comment
 *
 * @param enclosing_method_arguments
 *
 * @return return_type
 * @TODO delete or find out if it could be used.
 */
//TNC_Result TNC_IMUnit_ReceiveMessage(TNC_IMUnitID pImunitID, TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType) throw (TNCUtil::exception::ResultException);

/**
 * TODO_Comment
 *
 * @param enclosing_method_arguments
 *
 * @return return_type
 * @TODO delete or find out if it could be used.
 */
//TNC_Result TNC_IMUnit_BatchEnding(TNC_IMUnitID pImunitID, TNC_ConnectionID connectionID) throw (TNCUtil::exception::ResultException);

/**
 * TODO_Comment
 *
 * @param enclosing_method_arguments
 *
 * @return return_type
 * @TODO delete or find out if it could be used.
 */
//TNC_Result TNC_IMUnit_Terminate(TNC_IMUnitID pImunitID) throw (TNCUtil::exception::ResultException);

/**
 * Returns true, if TNC_IMV_Initialize() was already invoked by TNC server.
 *
 * @throw TNCUtil::exception::ResultException
 */
void checkIMUnitInitialized() throw (TNCUtil::exception::ResultException);

/**
 * Concatenates std::string s and TNC_UInt32 i.
 *
 * @param s
 * @param i
 *
 * @return std::string
 */
std::string operator+(const std::string & s, const TNC_UInt32 & i);

/**
 * TODO_Comment
 *
 * @param enclosing_method_arguments
 *
 * @return TNC_Version
 */
TNC_Version getVersion(TNC_Version min, TNC_Version max, TNC_Version support, TNC_Version *pOutActualVersion) throw (TNCUtil::exception::ResultException);

/**
 * Checks preconditions for all functions invoked by TNC server except TNC_IMV_Initialize().
 */
void checkPreconditions(TNC_IMUnitID id) throw (TNCUtil::exception::ResultException);

/**
 * Check parameter for TNC_..._ReciveMessage.
 *
 * @param id the IMUnitID
 * @param messageBuffer the message
 * @param messageLength the length of the message
 * @param messageType the msessageType
 *
 * @throw TNCUtil::exception::ResultException if a parameter is invalid
 */
void checkReciveMessageParameter(TNC_IMUnitID id, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType) throw (TNCUtil::exception::ResultException);

/**
 * This function returns a pointer to a TNC client/server function with the
 * specified name.
 *
 * @param functionName The name of the function that shall be looked up.
 * @param tncBindFunction The bindFunction from the client/server that has been previously
 *        provided by TNC_IMC/IMV_provideBindFunction.
 *
 * @return a pointer to the desired function.
 */
void * getTNCFunctionPointer(const char *functionName, TNC_BindFunctionPointer tncBindFunction) throw (TNCUtil::exception::ResultException);

/**
 * Get an IMV by connection id
 *
 * @param the connection id
 *
 * @return the IMV
 *
 * @throw TNCUtil::exception::ResultException if IMV not found
 */
TNCUtil::iml::AbstractIMV * getImvInstances(TNC_ConnectionID id) throw (TNCUtil::exception::ResultException);

/**
 * Get an IMC by connection id
 *
 * @param the connection id
 *
 * @return the IMC
 *
 * @throw TNCUtil::exception::ResultException if IMC not found
 */
TNCUtil::iml::AbstractIMC * getImcInstances(TNC_ConnectionID id) throw (TNCUtil::exception::ResultException);

/**
 * IMV/IMC ID assigned by TNC server
 */
extern TNC_IMUnitID imUnitID;

/**
 * This struct holds all the function pointers to the TNCC functions.
 */
struct TNCC_FunctionPointers{
	TNC_TNCC_ReportMessageTypesPointer reportMessageTypesPointer;
	TNC_TNCC_SendMessagePointer sendMessagePointer;
	TNC_TNCC_RequestHandshakeRetryPointer requestHandshakeRetryPointer;
	TNC_TNCC_BindFunctionPointer bindFunctionPointer;
};

/**
 * This struct holds all the function pointers to the TNCS functions.
 */
struct TNCS_FunctionPointers{
	TNC_TNCS_ReportMessageTypesPointer reportMessageTypesPointer;
	TNC_TNCS_SendMessagePointer sendMessagePointer;
	TNC_TNCS_RequestHandshakeRetryPointer requestHandshakeRetryPointer;
	TNC_TNCS_ProvideRecommendationPointer provideRecommendationPointer;
	TNC_TNCS_BindFunctionPointer bindFunctionPointer;
	TNC_TNCS_GetAttributePointer getAttributePointer;
	TNC_TNCS_SetAttributePointer setAttributePointer;
};

/**
 * The function pointer from TNC-Client
 */
extern struct TNCC_FunctionPointers tnccFunctionPointers;

/**
 * The function pointer from TNC-Server
 */
extern struct TNCS_FunctionPointers tncsFunctionPointers;

/**
 * Map with pointers to concrete IMV instances for a given connection ID.
 */
extern std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMV *> imvInstances;

/**
 * Map with pointers to concrete IMC instances for a given connection ID.
 */
extern std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMC *> imcInstances;

/**
 * Choosen API version for IMC/IMV
 */
extern TNC_Version actualVersion;

/**
 * True, if IMV has been initialized
 */
extern bool initialized;


/**
 * Is semaphore Initialize
 */
extern bool isSemaphoreInitialized;

/**
 * A class for syncronize threads. Create a Semaohore, if you want enter to a
 * critical area. If you want leaf the critical area, then destroy the object.
 * This class synchronized automatically the critical area.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>08.02.2008 - create class (mbs)</li>
 * </ul>
 *
 * @class Semaphore
 * @brief A class for syncronize threads.
 * @date 08.02.2008
 * @author Mike Steinmetz (mbs)
 */
class Semaphore {
public:
	/**
	 * Enter a critical area.
	 *
	 * @throw exception::SemaphoreResultException if an error on semaphore
	 */
	Semaphore() throw (TNCUtil::exception::ResultException);
	/**
	 * Leaf a critical area.
	 */
	virtual ~Semaphore();
};

}  // namespace iml

}  // namespace TNCUtil

#endif /*IFIMUNITIMPL_H_*/
