/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef IMUNITINFO_H_
#define IMUNITINFO_H_

#include "../tcg/tnc/tncifimc.h"
#include "../tcg/tnc/tncifimv.h"

#include <string>

namespace TNCUtil {

namespace iml {

/** @file
 * General information about an IMC/IMV.

 * <h3>Changelog:</h3>
 * <ul>
 *   <li>07.02.2008 - create class (mbs)</li>
 *   <li>06.01.2009 - added comment (ib)</li>
 * </ul>
 *
 * @date 07.02.2008
 * @author Mike Steinmetz (mbs)
 * @author Ingo Bente (ib)
 */
struct IMUnitInfo {
	/**
	 * List of message types for which the IMC/IMV wishes to receive messages.
	 * @see section 3.8.1 TNC_TNCS_ReportMessageTypes(...) in IF-IMV version 1.2
	 *      https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMV_v1_2_r8.pdf
	 * @see section 3.8.1 TNC_TNCS_ReportMessageTypes(...) in IF-IMC version 1.2
	 *      https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf
	 */
	TNC_MessageTypeList supportedTypes;

	/**
	 * Number of message types for which the IMC/IMV wishes to receive messages.
	 * @see section 3.8.1 TNC_TNCS_ReportMessageTypes(...) in IF-IMV version 1.2
	 *      https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMV_v1_2_r8.pdf
	 * @see section 3.8.1 TNC_TNCS_ReportMessageTypes(...) in IF-IMC version 1.2
	 *      https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf
	 */
	TNC_UInt32 typeCount;

	/**
	 * The name of the IMC/IMV. This name is used for logging purposes.
	 */
	std::string imUnitName;
};

}  // namespace iml

}  // namespace TNCUtil

#endif /*IMUNITINFO_H_*/
