/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#include "TNCC.h"
#include "IFIMUnitImpl.h"

namespace TNCUtil {

namespace iml {

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                      TNCC::TNCC(TNC_ConnectionID cid)                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCC::TNCC(TNC_ConnectionID cid):
	cid(cid)
{
	// intentionally left blank
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                               TNCC::~TNCC()                               *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCC::~TNCC()
{
	// intentionally left blank
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *  void TNCC::sendMessage(TNC_BufferReference, TNC_UInt32, TNC_MessageType) *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCC::sendMessage(TNC_BufferReference message, TNC_UInt32 messageLength, TNC_MessageType messageType) throw (TNCUtil::exception::ResultException)
{
	if (tnccFunctionPointers.sendMessagePointer == NULL)
		throw TNCUtil::exception::ResultException("Could not send a message, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tnccFunctionPointers.sendMessagePointer(imUnitID, cid, message, messageLength, messageType);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on send a message. Result: ")+result, result);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *             void TNCC::requestHandshakeRetry(TNC_RetryReason)             *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCC::requestHandshakeRetry(TNC_RetryReason reason) throw (TNCUtil::exception::ResultException)
{
	if (tnccFunctionPointers.requestHandshakeRetryPointer == NULL)
		throw TNCUtil::exception::ResultException("Could not requestHandshakeRetry, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tnccFunctionPointers.requestHandshakeRetryPointer(imUnitID, cid, reason);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on requestHandshakeRetry. Result: ")+result, result);
}

}  // namespace iml

}  // namespace TNCUtil
