/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef TNCC_H_
#define TNCC_H_

#include "../tcg/tnc/tncifimc.h"
#include "../exception/ResultException.h"

namespace TNCUtil {

namespace iml {

/**
 * Interfaceclass for the TNC-Client. With this class, you can call
 * functions on TNC-Client.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>09.02.2008 - create class (mbs)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 * </ul>
 *
 * @class TNCC
 * @brief Interfaceclass for the TNC-Client.
 * @date 09.02.2008
 * @author Mike Steinmetz (mbs)
 */
class TNCC
{
private:
	/**
	 * The connection ID
	 */
	TNC_ConnectionID cid;
public:
	/**
	 * Ctor.
	 *
	 * @param cid the connection ID
	 */
	TNCC(TNC_ConnectionID cid);

	/**
	 * Dtor.
	 */
	virtual ~TNCC();
//	virtual void reportMessageTypes() throw (exception::Exception);

	/**
	 * send a IMV/IMC message
	 *
	 * @param message The binary message you want to send.
	 * @param messageLength The length of message.
	 * @param messageType The type of message.
	 *
	 * @see <a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">TCG TNC IF-IMC Specification 1.2</a>
	 */
	virtual void sendMessage(TNC_BufferReference message, TNC_UInt32 messageLength, TNC_MessageType messageType) throw (TNCUtil::exception::ResultException);

	/**
	 * Call the TNCC and request to retry the TNC handshake.
	 *
	 * @param Reason the reason of handshake retry
	 *
	 * @see <a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">TCG TNC IF-IMC Specification 1.2</a>
	 */
	virtual void requestHandshakeRetry(TNC_RetryReason reason) throw (TNCUtil::exception::ResultException);
};

}  // namespace iml

}  // namespace TNCUtil

#endif /*TNCC_H_*/
