/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#include "TNCS.h"
#include "IFIMUnitImpl.h"

namespace TNCUtil {

namespace iml {

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                      TNCS::TNCS(TNC_ConnectionID cid)                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCS::TNCS(TNC_ConnectionID cid):
	cid(cid)
{
	// intentionally left blank
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                               TNCS::~TNCS()                               *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

TNCS::~TNCS()
{
	// intentionally left blank
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *  void TNCS::sendMessage(TNC_BufferReference, TNC_UInt32, TNC_MessageType) *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCS::sendMessage(TNC_BufferReference message, TNC_UInt32 messageLength, TNC_MessageType messageType) throw (TNCUtil::exception::ResultException)
{
	if (tncsFunctionPointers.sendMessagePointer == NULL)
		throw TNCUtil::exception::ResultException("Could not send a message, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tncsFunctionPointers.sendMessagePointer(imUnitID, cid, message, messageLength, messageType);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on send a message. Result: ")+result, result);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *             void TNCS::requestHandshakeRetry(TNC_RetryReason)             *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCS::requestHandshakeRetry(TNC_RetryReason reason) throw (TNCUtil::exception::ResultException)
{
	if (tncsFunctionPointers.requestHandshakeRetryPointer == NULL)
		throw TNCUtil::exception::ResultException("Could not requestHandshakeRetry, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tncsFunctionPointers.requestHandshakeRetryPointer(imUnitID, cid, reason);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on requestHandshakeRetry. Result: ")+result, result);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *  void TNCS::provideRecommendation(                                        *
 *      TNC_IMV_Action_Recommendation recommendation,                        *
 *      TNC_IMV_Evaluation_Result evaluation) throw (TNCUtil::exception::ResultException)   *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void TNCS::provideRecommendation(TNC_IMV_Action_Recommendation recommendation, TNC_IMV_Evaluation_Result evaluation) throw (TNCUtil::exception::ResultException)
{
	if (tncsFunctionPointers.provideRecommendationPointer == NULL)
		throw TNCUtil::exception::ResultException("Could not provide recommendation, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tncsFunctionPointers.provideRecommendationPointer(imUnitID, cid, recommendation, evaluation);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on provide recommendation. Result: ")+result, result);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *      void TNCS::getAttribute(TNC_AttributeID attributeID,                 *
 *           TNC_UInt32 bufferLength, TNC_BufferReference buffer,            *
 *           TNC_UInt32 *pOutValueLength) throw (TNCUtil::exception::ResultException)       *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCS::getAttribute(/*in*/ TNC_AttributeID attributeID, /*in*/ TNC_UInt32 bufferLength, /*out*/ TNC_BufferReference buffer, /*out*/ TNC_UInt32 *pOutValueLength) throw (TNCUtil::exception::ResultException)
{
	if (tncsFunctionPointers.getAttributePointer == NULL)
		throw TNCUtil::exception::ResultException("Could not get attribute, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tncsFunctionPointers.getAttributePointer(imUnitID, cid, attributeID, bufferLength, buffer, pOutValueLength);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on get attribute. Result: ")+result, result);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     bool TNCS::supportGetAttribute()                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool TNCS::supportGetAttribute()
{
	return tncsFunctionPointers.getAttributePointer != NULL;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *        void TNCS::setAttribute(TNC_AttributeID attributeID,               *
 *             TNC_UInt32 bufferLength, TNC_BufferReference buffer)          *
 *             throw (TNCUtil::exception::ResultException)                                  *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCS::setAttribute(TNC_AttributeID attributeID, TNC_UInt32 bufferLength, TNC_BufferReference buffer) throw (TNCUtil::exception::ResultException)
{
	if (tncsFunctionPointers.setAttributePointer == NULL)
		throw TNCUtil::exception::ResultException("Could not set attribute, because i haven't the functionpointer", TNC_RESULT_FATAL);

	TNC_Result result = tncsFunctionPointers.setAttributePointer(imUnitID, cid, attributeID, bufferLength, buffer);

	if (result != TNC_RESULT_SUCCESS)
		throw TNCUtil::exception::ResultException(std::string("Exception on set attribute. Result: ")+result, result);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     bool TNCS::supportSetAttribute()                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool TNCS::supportSetAttribute()
{
	return tncsFunctionPointers.setAttributePointer != NULL;
}

}  // namespace iml

}  // namespace TNCUtil
