/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef HOOK_H_
#define HOOK_H_

#include "IMUnitInfo.h"
#include "AbstractIMUnit.h"

namespace TNCUtil {

/** @file hook.h
 *
 * These header file declares all hook functions. They must be implemented by the developer
 * of the concrete IMC/IMV.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>07.02.2008 - create class (ib)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 * </ul>
 *
 * @date 07.02.2008
 * @author Ingo Bente (ib)
 * @author Mike Steinmetz (mbs)
 */
namespace iml {

/**
 * This function must be implemented by the concrete IMC/IMV. Can not be part
 * of AbstractIMUnit because it is called before any object is created.
 *
 * @return The corresponding IMUnitInfo.
 */
IMUnitInfo getIMUnitInfo();

/**
 * Helper method to create the concrete IMC/IMV instance after TNC_IMC/IMV_Initialize()
 * has been invoked by TNC client/server. Must be implemented by concrete IMC/IMV. Can
 * not be part of AbstractIMUnit because it is called before any object is created.
 *
 * @return A pointer to a new IMUnit instance.
 *
 * @param conID the ConnectionID
 */
AbstractIMUnit * createNewIMUnitInstance(TNC_ConnectionID conID);

}  // namespace iml

}  // namespace TNCUtil

#endif /*HOOK_H_*/
