/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 * 
 * http://tnc.inform.fh-hannover.de/
 */

#include "TNCIOHelper.h"

#include <iomanip>
#include "TNCLog.h"

namespace TNCUtil {

/*
 * @namespace TNCUtil::io
 * 
 * TNCIOHelper implementation. Provides helper methods for file and stream operations.
 *
 * @author Daniel Wuttke
 * @author Ingo Bente
 * @author Mike Steinmetz (mbs)
 * 
 * @date 13.06.2006
 * 
 * @todo Create class for TNCIOHelper instead of functions in namespace TNCIOHelper ?
 *        || Create cascading namespace TNCUtil::TNCIOHelper ?
 */
namespace io {

/* for logging */
#define CLASSNAME "TNCIOHelper"

#define MAX_LINE_LENGTH 512

	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *void printBinaryOut(const unsigned char* const, unsigned long, std::string,*
	 *                            std::ostream &cout)                            *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void printBinaryOut(const unsigned char * const packet, unsigned long packetLength, std::string text, std::ostream &cout) {
	
		std::string ascii;
		unsigned int i;
		
		cout << std::endl << text << "(" << std::dec << packetLength << "):";
		for (i = 0; i < packetLength; ++i) {
			if (i%16 == 0) {
				cout << "   " << ascii << std::endl
				     << " " << std::setw(4) << std::setfill('0') << std::hex << i << "   ";
				ascii = "";
			} else if (i%8 == 0) {
				ascii += ' ';
				cout << " ";
			}
			
			cout << std::setw(2) << std::setfill('0') << std::hex << static_cast<int>(packet[i]) << " ";
			if (packet[i]>=' ' && packet[i] <= 'z')
				ascii += static_cast<char>(packet[i]);
			else
				ascii += '.';
		}
		while (i%16) {
			if (i%8 == 0) {
				cout << " ";
			}
			cout << "   ";
			i++;
		}
		cout << "   " << ascii << std::endl;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *     void printBinaryOut(const char* const, unsigned long, std::string,    *
	 *                            std::ostream &cout)                            *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void printBinaryOut(const char * const packet, unsigned long packetLength, std::string text, std::ostream &cout) {
		printBinaryOut(reinterpret_cast<const unsigned char * const>(packet), packetLength, text, cout);
	}

	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *              std::ifstream * openFile(std::string filename)               *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::ifstream * openFile(std::string filename, bool errOut) {
	    std::ifstream *in = new std::ifstream();
	    in -> open(filename.c_str(), std::ios::in);
	    if (!(*in)) {
	    	if (errOut)
	    		std::cerr << "Could not open file " << filename << "!" << std::endl;
	        delete in;
	        in = NULL;
	    }
	    return in;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *                     void closeFile(std::ifstream *in)                     *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void closeFile(std::ifstream *in) {
		if(in == NULL)
		{
			std::cerr << "Could not close file. std::ifstream == NULL !" << std::endl;
		} else 
		{
			in->close();
			delete in;		
		}
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *         bool readLine(std::istream *in, char *buf, const int size)        *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	bool readLine(std::istream *in, char *buf, const int size) {
		if(in == NULL)
		{
			std::cerr << "Could not read file. std::istream == NULL !" << std::endl;
			return false;
		} // else
	    bool ret = in->getline(buf, size);
	    if (ret && buf[strlen(buf) - 1] == '\r') {
	        buf[strlen(buf) - 1] = '\0';
	    }
	    return ret;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *        bool readProperty(std::ifstream *in, char *buf,                    *
	 *                                const int size,                            *
	 *                                const char *propertyName)                  *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	bool readProperty(std::ifstream *in, char *buf, const int size, const char *propertyName) {
		//std::cout << "readProperty" << std::endl;
	    bool ret = false;
	
		in->clear();
		in->seekg(0, std::ios::beg);
		while (readLine(in, buf, size)) {
	        std::string line = std::string(buf);
	        u_int pos = line.find_first_of('=');
	        if (pos != std::string::npos) {
	            std::string key = line.substr(0, pos);
	            key = trim(key);
	            std::string value = line.substr(pos + 1, line.size());
	            value = trim(value);
	            //std::cout << key << " --> " << value << std::endl;
	            //std::cout << key << " == " << propertyName << std::endl;
	            //std::cout << key.compare(propertyName) << std::endl;
	            if (!key.compare(propertyName)) {
	            	//std::cout << "found!" << std::endl;
	                strncpy(buf, value.c_str(), size);
	                ret = true;
	                break;
	            }
	        }
	    }
	    return ret;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *       std::vector<std::pair<std::string,std::string>>                     * 
	 *                               readAllProperties(std::ifstream*)           *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::vector<std::pair<std::string,std::string> > readAllProperties(std::istream *in) {
		TNCLog &log = TNCLog::getLog("TNCIOHelper", "readAllProperties");
	
	    std::vector<std::pair<std::string,std::string> > properties;
	    
		in->clear();
		in->seekg(0, std::ios::beg);
		
		unsigned int buf_len = 1000;
		char buf[buf_len];
		
		while (readLine(in, buf, buf_len)) {
	        std::string line = trim(std::string(buf));
	        
			/* ignore comments */
			if (line.size()<1 || line.at(0)=='#')
				continue;
	        
	        u_int pos = line.find_first_of('=');
	        
	        /* ignore illegal lines */
	        if (pos == std::string::npos) {
	        	log.warn("Ignore illegal line");
	        	continue;
	        }
	        
            /* get key */
	        std::string key = line.substr(0, pos);
            key = trim(key);
            
            /* get value */
            std::string value = line.substr(pos + 1, line.size());
            value = trim(value);
            
            /* debug */
            log.debug("Read Key:\"%s\" Value:\"%s\"", key.c_str(), value.c_str());
            
            /* add key-value-pair */
            properties.push_back(std::pair<std::string, std::string>(key, value));
	    }
		
	    return properties;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *             std::string trim(std::string const& source,                   *
	 *                                    char const* delims)                    *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::string trim(std::string const& source, char const* delims) {
	  std::string result(source);
	  std::string::size_type index = result.find_last_not_of(delims);
	  if(index != std::string::npos)
	    result.erase(++index);
	
	  index = result.find_first_not_of(delims);
	  if(index != std::string::npos)
	    result.erase(0, index);
	  else
	    result.erase();
	  return result;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *             bool read(std::ifstream *in, char *buf,                       *
	 *                             const int size, char delimiter)               *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	bool read(std::istream *in, char *buf, const int size, char delimiter) {
		if(in == NULL)
		{
			std::cerr << "Could not read file. std::ifstream == NULL !" << std::endl;
			return false;
		} // else
	    bool ret = in->get(buf, size, delimiter);
	    // skip delimiter
	    in->get();
	    return ret;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *             std::ofstream * openOutputFile(std::string filename)          *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::ofstream * openOutputFile(std::string filename) {
	    std::ofstream *out = new std::ofstream();
	    out->open(filename.c_str(), std::ios::out);
	    if (!(*out)) {
	        std::cerr << "Could not open file " << filename << "!" << std::endl;
	        delete out;
	        out = NULL;
	    }
	    return out;
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *                      void closeFile(std::ofstream *out)                   *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void closeFile(std::ofstream *out) {
		if(out == NULL)
		{
			std::cerr << "Could not close file. std::ofstream == NULL !" << std::endl;
		} else
		{
			out->close();
			delete out;		
		}
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 *        void write(std::ofstream *out, const char *buf,                    *
	 *                         const int size)                                   *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	void write(std::ofstream *out, const char *buf, const int size) {
		if(out == NULL)
		{
			std::cerr << "Could not write file. std::ofstream == NULL !" << std::endl;
		} else
		{
		    out->write(buf, size);
		}
	}
	
	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 *                                                                           *
	 * std::vector<std::vector<std::string> > *parseTncConfigFile(const char *)  *
	 *                                     throw (TNCUtil::exception::Exception) *
	 *                                                                           *
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
	std::vector<std::string> *parseTncConfigFile(const char * file) throw (TNCUtil::exception::Exception)
	{
		TNCLog &log = TNCLog::getLog(CLASSNAME, "parseTncConfigFile");
		std::ifstream tnc_config(file);
		char line[MAX_LINE_LENGTH];
		std::vector<std::string> *ret = new std::vector<std::string>();
		std::vector<std::string> lines;
		
		if(!tnc_config){
			throw TNCUtil::exception::Exception("Could not open file %s", file);
		}
		
		log.info("Read tnc_config file: %s", file);
			
		// fill all lines to vector
		while(!tnc_config.eof()){
			tnc_config.getline(line, MAX_LINE_LENGTH);
			if(line != NULL){
				lines.push_back(std::string(line));				
			}
		}
		
		// close file
		tnc_config.close();
		
		// evaluate all lines
		for(std::vector<std::string>::iterator the_iterator = lines.begin();
		    the_iterator != lines.end();
		    the_iterator++){
			// skip comments
			if(!the_iterator->compare(0, 1, "#")){
				log.debug("Skipping comment: " + *the_iterator);
			}
			// skip empty lines
			else if(the_iterator->size() == 0){
				log.debug("Skipping empty line.");
			}
			// process IMV entry
			else if(!the_iterator->compare(0, 4, "IMV ")){
				// find first "
				std::string::size_type start = the_iterator->find( '"');
				// find second "
				std::string::size_type end = the_iterator->rfind( '"');
				// get substring between first " and second "
				std::string imvName = the_iterator->substr(start + 1, end - start - 1);
				// get SO path
				std::string imvSO = the_iterator->substr(end + 2);
				ret->push_back(imvName);
				ret->push_back(imvSO);
				log.debug("Adding IMV: imvName = " + imvName + ", imvSO = " + imvSO);
			}
			// skip invalid lines
			else if(!the_iterator->compare(0, 4, "IMC ")){
				log.debug("Skipping IMC line.");
			}
			else {
				log.error("Invalid line: " + *the_iterator);			
			}
		}
		return ret;
	}

}  // namespace io

}  // namespace TNCUtil
