/*
 * This software is Copyright (C) 2006 FH Hannover
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to share the
 * source code of your application with everyone you distribute it to, and you
 * also want to give them the right to share who uses it. If you wish to use
 * TNC@FHH under Open Source Licensing, you must contribute all your source code
 * to the open source community in accordance with the GPL Version 2 when your
 * application is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are creating
 * proprietary applications and you are not prepared to distribute and share the
 * source code of your application. Contact tnc@inform.fh-hannover.de for details.
 */

/*
 * TNCLog implementation.
 *
 * Author: Daniel Wuttke
 * Date: 09/04/2006
 * 
 */
#include "TNCLog.h"

// Default property file name
#ifndef TNC_LOG_PROPERTY_FILE
#ifdef __WINDOWS__
#define TNC_LOG_PROPERTY_FILE "C:/WINDOWS/tnc_log.properties"
#elif __LINUX__
#define TNC_LOG_PROPERTY_FILE "/etc/tnc_log.properties"
#else
#define TNC_LOG_PROPERTY_FILE "tnc_log.properties"
#endif
#endif


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNCLog::TNCLog(const char *propertyFile, const char *className,           *
 *                const char *methodName)                                    *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCLog::TNCLog(const char *propertyFile, const char *className, const char *methodName) :
    className(className), methodName(methodName), debugLevel(TNC_DEFAULT_LOG_LEVEL) {

    //cout << "Constructor TNCLog" << endl;

    if (propertyFile == NULL) {
        propertyFile = TNC_LOG_PROPERTY_FILE;
    }
    std::ifstream *in = TNCUtil::io::openFile(propertyFile, true);
    if (in != NULL) {
        char buf[LOG_BUFFER_SIZE_DEFAULT];
        if (TNCUtil::io::readProperty(in, buf, LOG_BUFFER_SIZE_DEFAULT, className)) {
            //std::cout << "Class found" << std::endl;
            this->debugLevel = strtol(buf, NULL, 10);
        } else if (TNCUtil::io::readProperty(in, buf, LOG_BUFFER_SIZE_DEFAULT, "default")) {
        	//std::cout << "default found" << std::endl;
            this->debugLevel = strtol(buf, NULL, 10);
        }
        TNCUtil::io::closeFile(in);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                            TNCLog::~TNCLog()                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCLog::~TNCLog() {
    //cout << "Destructor TNCLog: " << className << "::" << methodName << " | Debug-Level: " << debugLevel << endl;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::debug(std::string &message)                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::debug(const std::string &message) {
    debug(message.c_str());
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::info(std::string &message)                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::info(const std::string &message) {
    info(message.c_str());
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::warn(std::string &message)                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::warn(const std::string &message) {
    warn(message.c_str());
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::error(std::string &message)                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::error(const std::string &message) {
    error(message.c_str());
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::debug(const char *message, ...)                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::debug(const char *message, ...) {
    if (isDebug()) {
        va_list l;
        va_start(l, message);
        char *buf = (char *) calloc(logBufferSize, sizeof(char));
        vsnprintf(buf, logBufferSize, message, l);
        *out << "<D> " << className << "::" << methodName << ": " << buf << std::endl;
        free(buf);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::info(const char *message, ...)                  *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::info(const char *message, ...) {
    if (isInfo()) {
        va_list l;
        va_start(l, message);
        char *buf = (char *) calloc(logBufferSize, sizeof(char));
        vsnprintf(buf, logBufferSize, message, l);
        *out << "<I> " << className << "::" << methodName << ": " << buf << std::endl;
        free(buf);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::warn(const char *message, ...)                  *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::warn(const char *message, ...) {
    if (isWarn()) {
        va_list l;
        va_start(l, message);
        char *buf = (char *) calloc(logBufferSize, sizeof(char));
        vsnprintf(buf, logBufferSize, message, l);
        *out << "<W> " << className << "::" << methodName << ": " << buf << std::endl;
        free(buf);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::error(const char *message, ...)                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::error(const char *message, ...) {
    if (isError()) {
        va_list l;
        va_start(l, message);
        char *buf = (char *) calloc(logBufferSize, sizeof(char));
        vsnprintf(buf, logBufferSize, message, l);
        *err << "<E> " << className << "::" << methodName << ": " << buf << std::endl;
        free(buf);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                          bool TNCLog::isDebug()                           *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool TNCLog::isDebug() {
    return this->debugLevel >= TNC_LOG_LEVEL_DEBUG;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                          bool TNCLog::isInfo()                            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool TNCLog::isInfo() {
    return this->debugLevel >= TNC_LOG_LEVEL_INFO;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                          bool TNCLog::isWarn()                            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool TNCLog::isWarn() {
    return this->debugLevel >= TNC_LOG_LEVEL_WARN;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                          bool TNCLog::isError()                           *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
bool TNCLog::isError(){
    return this->debugLevel >= TNC_LOG_LEVEL_ERROR;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNCLog & TNCLog::getLog(const char *className, const char *methodName,    *
 *                         const char *propertyFile)                         *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCLog & TNCLog::getLog(const char *className, const char *methodName, const char *propertyFile) {
    std::string key = className;
    key.append(methodName);
    TNCLog *logPointer = logInstances[key];
    if (logPointer == NULL) {
        logPointer = new TNCLog(propertyFile, className, methodName);
        logInstances[key] = logPointer;
    }
    logPointer -> debug("Methoden-Start");

    return *logPointer;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                    void TNCLog::deleteLogs()                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::deleteLogs() {
    std::map<std::string, TNCLog*>::iterator iter;
    for (iter = logInstances.begin(); iter != logInstances.end(); iter++) {
        std::pair<std::string, TNCLog*> mapPair = *iter;
        delete(mapPair.second);
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::setOutputStream(std::ostream &out)              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::setOutputStream(std::ostream &out) {
    TNCLog::out = &out;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *              void TNCLog::setErrOutputStream(std::ostream &err)           *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::setErrOutputStream(std::ostream &err) {
    TNCLog::err = &err;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                void TNCLog::setLogBufferSize(int bufferSize)              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void TNCLog::setLogBufferSize(int bufferSize) {
    TNCLog::logBufferSize = bufferSize;
}

std::map<std::string, TNCLog*> TNCLog::logInstances = std::map<std::string, TNCLog*>();
std::ostream *TNCLog::out = &std::cout;
std::ostream *TNCLog::err = &std::cerr;
int TNCLog::logBufferSize = LOG_BUFFER_SIZE_DEFAULT;
