#ifdef __WINDOWS__
/*
 * This software is Copyright (C) 2006 FH Hannover
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to share the
 * source code of your application with everyone you distribute it to, and you
 * also want to give them the right to share who uses it. If you wish to use
 * TNC@FHH under Open Source Licensing, you must contribute all your source code
 * to the open source community in accordance with the GPL Version 2 when your
 * application is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are creating
 * proprietary applications and you are not prepared to distribute and share the
 * source code of your application. Contact tnc@inform.fh-hannover.de for details.
 */

/**
 * Header file for TNCRegistryHelper. Provides helper methods to access
 * the Windows Registry.
 * 
 * @author: Daniel Wuttke
 * @date: 18/05/2006 
 */
#ifndef TNCREGISTRYHELPER_H_
#define TNCREGISTRYHELPER_H_

/* Must be included first! */
#include <windows.h>

#include <string>
#include <vector>

namespace TNCRegistryHelper {
    
    char * getRegString(HKEY preservedKey, const char *subKey, const char *valueName, char *defaultValue = NULL);         
    
    DWORD getRegNumber(HKEY preservedKey, const char *subKey, const char *valueName, DWORD defaultValue = 0);

    int getRegBinary(HKEY preservedKey, const char *subKey, const char *valueName, BYTE **buffer);
    
    std::vector<std::string> * getSubKeys(HKEY preservedKey, const char *subKey);
    
    /**
     * Maps the parameter 'key' to an appropriate char*.
     * @param key The key that shall be mapped.
     * 
     * @return The HKEY parameter as char*.
     */
    const char * getHKEYName(const HKEY key);
}

#endif /*TNCREGISTRYHELPER_H_*/
#endif /*__WINDOWS__*/
